@@ -1,9 +1,7 @@
 /*
- * $Xorg: xdpyinfo.c,v 1.5 2001/02/09 02:05:41 xorgcvs Exp $
- * 
  * xdpyinfo - print information about X display connection
  *
- * 
+ *
 Copyright 1988, 1998  The Open Group
 Copyright 2005 Hitachi, Ltd.
 
@@ -30,8 +28,6 @@ in this Software without prior written authorization from The Open Group.
  * Author:  Jim Fulton, MIT X Consortium
  */
 
-/* $XFree86: xc/programs/xdpyinfo/xdpyinfo.c,v 3.29 2003/04/14 20:38:10 herrb Exp $ */
-
 #ifdef HAVE_CONFIG_H
 # include "config.h"
 # if HAVE_X11_EXTENSIONS_MULTIBUF_H
@@ -80,12 +76,13 @@ in this Software without prior written authorization from The Open Group.
 #  define DMX
 # endif
 
-# if HAVE_X11_EXTENSIONS_PRINT_H
-#  define INCLUDE_XPRINT_SUPPORT
-# endif
-
 #endif
 
+#ifdef WIN32
+#include <X11/Xwindows.h>
+#endif
+
+#include <X11/Xlib-xcb.h>
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #ifdef MULTIBUFFER
@@ -140,21 +137,15 @@ in this Software without prior written authorization from The Open Group.
 #ifdef DMX
 #include <X11/extensions/dmxext.h>
 #endif
-#ifdef INCLUDE_XPRINT_SUPPORT
-#include <X11/extensions/Print.h>
-#endif /* INCLUDE_XPRINT_SUPPORT */
 #include <X11/Xos.h>
 #include <stdio.h>
 #include <stdlib.h>
 
-/* Turn a NULL pointer string into an empty string */
-#define NULLSTR(x) (((x)!=NULL)?(x):(""))
-
 static char *ProgramName;
 static Bool queryExtensions = False;
 
 static int
-silent_errors(Display *dpy, XErrorEvent *ev)
+silent_errors(_X_UNUSED Display *dpy, _X_UNUSED XErrorEvent *ev)
 {
     return 0;
 }
@@ -165,7 +156,7 @@ static int print_event_mask(char *buf, int lastcol, int indent, long mask);
 
 static int StrCmp(const void *a, const  void *b)
 {
-    return strcmp(*(char **)a, *(char **)b);
+    return strcmp(*(const char * const *)a, *(const char * const *)b);
 }
 
 static void
@@ -177,22 +168,53 @@ print_extension_info(Display *dpy)
     printf ("number of extensions:    %d\n", n);
 
     if (extlist) {
-	register int i;
-	int opcode, event, error;
+	int i;
 
 	qsort(extlist, n, sizeof(char *), StrCmp);
-	for (i = 0; i < n; i++) {
-	    if (!queryExtensions) {
+
+	if (!queryExtensions) {
+	    for (i = 0; i < n; i++) {
 		printf ("    %s\n", extlist[i]);
-		continue;
 	    }
-	    XQueryExtension(dpy, extlist[i], &opcode, &event, &error);
-	    printf ("    %s  (opcode: %d", extlist[i], opcode);
-	    if (event)
-		printf (", base event: %d", event);
-	    if (error)
-		printf (", base error: %d", error);
-	    printf(")\n");
+	} else {
+	    xcb_connection_t *xcb_conn = XGetXCBConnection (dpy);
+	    xcb_query_extension_cookie_t *qe_cookies;
+
+	    qe_cookies = calloc(n, sizeof(xcb_query_extension_cookie_t));
+	    if (!qe_cookies) {
+		perror ("calloc failed to allocate memory for extensions");
+		return;
+	    }
+
+	    /*
+	     * Generate all extension queries at once, so they can be
+	     * sent to the xserver in a single batch
+	     */
+	    for (i = 0; i < n; i++) {
+		qe_cookies[i] = xcb_query_extension (xcb_conn,
+						     strlen(extlist[i]),
+						     extlist[i]);
+	    }
+
+	    /*
+	     * Start processing replies as they come in.
+	     * The first call will flush the queue to the server, then
+	     * each one will wait, if needed, for its reply.
+	     */
+	    for (i = 0; i < n; i++) {
+		xcb_query_extension_reply_t *rep
+		    = xcb_query_extension_reply(xcb_conn, qe_cookies[i], NULL);
+
+		printf ("    %s  (opcode: %d", extlist[i], rep->major_opcode);
+		if (rep->first_event)
+		    printf (", base event: %d", rep->first_event);
+		if (rep->first_error)
+		    printf (", base error: %d", rep->first_error);
+		printf (")\n");
+
+		free (rep);
+	    }
+	    free (qe_cookies);
 	}
 	/* do not free, Xlib can depend on contents being unaltered */
 	/* XFreeExtensionList (extlist); */
@@ -203,7 +225,7 @@ static void
 print_display_info(Display *dpy)
 {
     char dummybuf[40];
-    char *cp;
+    const char *cp;
     int minkeycode, maxkeycode;
     int i, n;
     long req_size;
@@ -310,8 +332,9 @@ print_display_info(Display *dpy)
     switch (BitmapBitOrder (dpy)) {
       case LSBFirst:    cp = "LSBFirst"; break;
       case MSBFirst:    cp = "MSBFirst"; break;
-      default:    
-	sprintf (dummybuf, "unknown order %d", BitmapBitOrder (dpy));
+      default:
+	snprintf (dummybuf, sizeof(dummybuf),
+                  "unknown order %d", BitmapBitOrder (dpy));
 	cp = dummybuf;
 	break;
     }
@@ -321,8 +344,9 @@ print_display_info(Display *dpy)
     switch (ImageByteOrder (dpy)) {
       case LSBFirst:    cp = "LSBFirst"; break;
       case MSBFirst:    cp = "MSBFirst"; break;
-      default:    
-	sprintf (dummybuf, "unknown order %d", ImageByteOrder (dpy));
+      default:
+	snprintf (dummybuf, sizeof(dummybuf),
+                  "unknown order %d", ImageByteOrder (dpy));
 	cp = dummybuf;
 	break;
     }
@@ -386,7 +410,7 @@ static void
 print_visual_info(XVisualInfo *vip)
 {
     char errorbuf[40];			/* for sprintfing into */
-    char *class = NULL;			/* for printing */
+    const char *class = NULL;		/* for printing */
 
     switch (vip->class) {
       case StaticGray:    class = "StaticGray"; break;
@@ -395,8 +419,8 @@ print_visual_info(XVisualInfo *vip)
       case PseudoColor:    class = "PseudoColor"; break;
       case TrueColor:    class = "TrueColor"; break;
       case DirectColor:    class = "DirectColor"; break;
-      default:    
-	sprintf (errorbuf, "unknown class %d", vip->class);
+      default:
+	snprintf (errorbuf, sizeof(errorbuf), "unknown class %d", vip->class);
 	class = errorbuf;
 	break;
     }
@@ -404,7 +428,7 @@ print_visual_info(XVisualInfo *vip)
     printf ("  visual:\n");
     printf ("    visual id:    0x%lx\n", vip->visualid);
     printf ("    class:    %s\n", class);
-    printf ("    depth:    %d plane%s\n", vip->depth, 
+    printf ("    depth:    %d plane%s\n", vip->depth,
 	    vip->depth == 1 ? "" : "s");
     if (vip->class == TrueColor || vip->class == DirectColor)
 	printf ("    available colormap entries:    %d per subfield\n",
@@ -418,48 +442,6 @@ print_visual_info(XVisualInfo *vip)
 	    vip->bits_per_rgb);
 }
 
-/* xc/programs/twm/twm.c has a copy of |hasExtension()|, please
- * keep both versions in sync... */
-static
-Bool hasExtension(Display *dpy, char *extname)
-{
-  int    num_extensions,
-         i;
-  char **extensions;
-  extensions = XListExtensions(dpy, &num_extensions);
-  for (i = 0; i < num_extensions &&
-         (strcmp(extensions[i], extname) != 0); i++);
-  XFreeExtensionList(extensions);
-  return i != num_extensions;
-}
-
-#ifdef INCLUDE_XPRINT_SUPPORT
-/* xc/programs/twm/twm.c has a copy of |IsPrintScreen()|, please
- * keep both versions in sync... */
-static
-Bool IsPrintScreen(Screen *s)
-{
-    Display *dpy = XDisplayOfScreen(s);
-    int      i;
-
-    /* Check whether this is a screen of a print DDX */
-    if (hasExtension(dpy, XP_PRINTNAME)) {
-        Screen **pscreens;
-        int      pscrcount;
-
-        pscreens = XpQueryScreens(dpy, &pscrcount);
-        for( i = 0 ; (i < pscrcount) && pscreens ; i++ ) {
-            if (s == pscreens[i]) {
-                XFree(pscreens);
-                return True;
-            }
-        }
-        XFree(pscreens);                      
-    }
-    return False;
-}
-#endif /* INCLUDE_XPRINT_SUPPORT */
-
 static void
 print_screen_info(Display *dpy, int scr)
 {
@@ -469,11 +451,10 @@ print_screen_info(Display *dpy, int scr)
     int nvi;				/* number of elements returned */
     int i;				/* temp variable: iterator */
     char eventbuf[80];			/* want 79 chars per line + nul */
-    static char *yes = "YES", *no = "NO", *when = "WHEN MAPPED";
+    static const char *yes = "YES", *no = "NO", *when = "WHEN MAPPED";
     double xres, yres;
     int ndepths = 0, *depths = NULL;
     unsigned int width, height;
-    Bool isPrintScreen = False; /* Initialise this if |INCLUDE_XPRINT_SUPPORT| is not set */
 
     /*
      * there are 2.54 centimeters to an inch; so there are 25.4 millimeters.
@@ -483,40 +464,24 @@ print_screen_info(Display *dpy, int scr)
      *         = N * 25.4 pixels / M inch
      */
 
-    xres = ((((double) DisplayWidth(dpy,scr)) * 25.4) / 
+    xres = ((((double) DisplayWidth(dpy,scr)) * 25.4) /
 	    ((double) DisplayWidthMM(dpy,scr)));
-    yres = ((((double) DisplayHeight(dpy,scr)) * 25.4) / 
+    yres = ((((double) DisplayHeight(dpy,scr)) * 25.4) /
 	    ((double) DisplayHeightMM(dpy,scr)));
 
     printf ("\n");
     printf ("screen #%d:\n", scr);
-
-#ifdef INCLUDE_XPRINT_SUPPORT
-    /* Check whether this is a screen of a print DDX */
-    isPrintScreen = IsPrintScreen(s);
-    printf ("  print screen:    %s\n", isPrintScreen?"yes":"no");
-#endif /* INCLUDE_XPRINT_SUPPORT */
-
-    if (isPrintScreen) {
-        /* Print resolution is set on a per-printer basis (per-document
-         * or per-page), the screen itself has no "default" resolution */
-        printf ("  maximum dimensions:    %dx%d pixels\n",
-	        XDisplayWidth (dpy, scr),  XDisplayHeight (dpy, scr));
-    }
-    else
-    {
-        printf ("  dimensions:    %dx%d pixels (%dx%d millimeters)\n",
-	        XDisplayWidth (dpy, scr),  XDisplayHeight (dpy, scr),
-	        XDisplayWidthMM(dpy, scr), XDisplayHeightMM (dpy, scr));
-        printf ("  resolution:    %dx%d dots per inch\n", 
-	        (int) (xres + 0.5), (int) (yres + 0.5));
-    }
+    printf ("  dimensions:    %dx%d pixels (%dx%d millimeters)\n",
+	    XDisplayWidth (dpy, scr),  XDisplayHeight (dpy, scr),
+	    XDisplayWidthMM(dpy, scr), XDisplayHeightMM (dpy, scr));
+    printf ("  resolution:    %dx%d dots per inch\n",
+	    (int) (xres + 0.5), (int) (yres + 0.5));
     depths = XListDepths (dpy, scr, &ndepths);
     if (!depths) ndepths = 0;
     printf ("  depths (%d):    ", ndepths);
     for (i = 0; i < ndepths; i++) {
 	printf ("%d", depths[i]);
-	if (i < ndepths - 1) { 
+	if (i < ndepths - 1) {
 	    putchar (',');
 	    putchar (' ');
 	}
@@ -551,7 +516,7 @@ print_screen_info(Display *dpy, int scr)
     viproto.screen = scr;
     vip = XGetVisualInfo (dpy, VisualScreenMask, &viproto, &nvi);
     printf ("  number of visuals:    %d\n", nvi);
-    printf ("  default visual id:  0x%lx\n", 
+    printf ("  default visual id:  0x%lx\n",
 	    XVisualIDFromVisual (DefaultVisual (dpy, scr)));
     for (i = 0; i < nvi; i++) {
 	print_visual_info (vip+i);
@@ -567,7 +532,7 @@ print_screen_info(Display *dpy, int scr)
 #define MASK_NAME_WIDTH 25
 
 static struct _event_table {
-    char *name;
+    const char *name;
     long value;
 } event_table[] = {
     { "KeyPressMask             ", KeyPressMask },
@@ -597,7 +562,7 @@ static struct _event_table {
     { "OwnerGrabButtonMask      ", OwnerGrabButtonMask },
     { NULL, 0 }};
 
-static int                      
+static int
 print_event_mask(char *buf,     /* string to write into */
                  int lastcol,   /* strlen(buf)+1 */
                  int indent,    /* amount by which to indent */
@@ -634,7 +599,7 @@ print_event_mask(char *buf,     /* string to write into */
 }
 
 static void
-print_standard_extension_info(Display *dpy, char *extname, 
+print_standard_extension_info(Display *dpy, const char *extname,
 			      int majorrev, int minorrev)
 {
     int opcode, event, error;
@@ -652,13 +617,12 @@ print_standard_extension_info(Display *dpy, char *extname,
 
 #ifdef MULTIBUFFER
 static int
-print_multibuf_info(Display *dpy, char *extname)
+print_multibuf_info(Display *dpy, const char *extname)
 {
     int i, j;			/* temp variable: iterator */
     int nmono, nstereo;		/* count */
     XmbufBufferInfo *mono_info = NULL, *stereo_info = NULL; /* arrays */
-    static char *fmt = 
-	"    visual id, max buffers, depth:    0x%lx, %d, %d\n";
+#define MULTIBUF_FMT "    visual id, max buffers, depth:    0x%lx, %d, %d\n"
     int scr = 0;
     int majorrev, minorrev;
 
@@ -677,12 +641,12 @@ print_multibuf_info(Display *dpy, char *extname)
 	} else {
 	    printf ("  screen %d number of mono multibuffer types:    %d\n", i, nmono);
 	    for (j = 0; j < nmono; j++) {
-		printf (fmt, mono_info[j].visualid, mono_info[j].max_buffers,
-			mono_info[j].depth);
+		printf (MULTIBUF_FMT, mono_info[j].visualid,
+			mono_info[j].max_buffers, mono_info[j].depth);
 	    }
 	    printf ("  number of stereo multibuffer types:    %d\n", nstereo);
 	    for (j = 0; j < nstereo; j++) {
-		printf (fmt, stereo_info[j].visualid,
+		printf (MULTIBUF_FMT, stereo_info[j].visualid,
 			stereo_info[j].max_buffers, stereo_info[j].depth);
 	    }
 	    if (mono_info) XFree ((char *) mono_info);
@@ -694,7 +658,7 @@ print_multibuf_info(Display *dpy, char *extname)
 #endif
 
 static int
-print_xtest_info(Display *dpy, char *extname)
+print_xtest_info(Display *dpy, const char *extname)
 {
     int majorrev, minorrev, foo;
 
@@ -705,7 +669,7 @@ print_xtest_info(Display *dpy, char *extname)
 }
 
 static int
-print_sync_info(Display *dpy, char *extname)
+print_sync_info(Display *dpy, const char *extname)
 {
     int majorrev, minorrev;
     XSyncSystemCounter *syscounters;
@@ -729,7 +693,7 @@ print_sync_info(Display *dpy, char *extname)
 }
 
 static int
-print_shape_info(Display *dpy, char *extname)
+print_shape_info(Display *dpy, const char *extname)
 {
     int majorrev, minorrev;
 
@@ -741,7 +705,7 @@ print_shape_info(Display *dpy, char *extname)
 
 #ifdef XFreeXDGA
 static int
-print_dga_info(Display *dpy, char *extname)
+print_dga_info(Display *dpy, const char *extname)
 {
     unsigned int offset;
     int majorrev, minorrev, width, bank, ram, flags;
@@ -750,7 +714,7 @@ print_dga_info(Display *dpy, char *extname)
 	return 0;
     print_standard_extension_info(dpy, extname, majorrev, minorrev);
 
-    if (!XF86DGAQueryDirectVideo(dpy, DefaultScreen(dpy), &flags) 
+    if (!XF86DGAQueryDirectVideo(dpy, DefaultScreen(dpy), &flags)
 	|| ! (flags & XF86DGADirectPresent) )
     {
 	printf("  DGA not available on screen %d.\n", DefaultScreen(dpy));
@@ -772,11 +736,11 @@ print_dga_info(Display *dpy, char *extname)
 #endif
 
 #ifdef XF86VIDMODE
-#define V_PHSYNC        0x001 
+#define V_PHSYNC        0x001
 #define V_NHSYNC        0x002
 #define V_PVSYNC        0x004
 #define V_NVSYNC        0x008
-#define V_INTERLACE     0x010 
+#define V_INTERLACE     0x010
 #define V_DBLSCAN       0x020
 #define V_CSYNC         0x040
 #define V_PCSYNC        0x080
@@ -812,7 +776,7 @@ print_XF86VidMode_modeline(
 }
 
 static int
-print_XF86VidMode_info(Display *dpy, char *extname)
+print_XF86VidMode_info(Display *dpy, const char *extname)
 {
     int majorrev, minorrev, modecount, dotclock, i;
     XF86VidModeMonitor monitor;
@@ -884,18 +848,20 @@ print_XF86VidMode_info(Display *dpy, char *extname)
 
 #ifdef XF86MISC
 
-static char *kbdtable[] = { "Unknown", "84-key", "101-key", "Other", "Xqueue" };
-static char *msetable[] = { "None", "Microsoft", "MouseSystems", "MMSeries",
+static const char *kbdtable[] = {
+		     "Unknown", "84-key", "101-key", "Other", "Xqueue" };
+static const char *msetable[] = {
+		     "None", "Microsoft", "MouseSystems", "MMSeries",
 		     "Logitech", "BusMouse", "Mouseman", "PS/2", "MMHitTab",
 		     "GlidePoint", "IntelliMouse", "ThinkingMouse",
 		     "IMPS/2", "ThinkingMousePS/2", "MouseManPlusPS/2",
 		     "GlidePointPS/2", "NetMousePS/2", "NetScrollPS/2",
 		     "SysMouse", "Auto" };
-static char *flgtable[] = { "None", "ClearDTR", "ClearRTS",
-		     "ClearDTR and ClearRTS" };
+static const char *flgtable[] = {
+		     "None", "ClearDTR", "ClearRTS", "ClearDTR and ClearRTS" };
 
 static int
-print_XF86Misc_info(Display *dpy, char *extname)
+print_XF86Misc_info(Display *dpy, const char *extname)
 {
     int majorrev, minorrev;
     XF86MiscMouseSettings mouseinfo;
@@ -946,7 +912,7 @@ print_XF86Misc_info(Display *dpy, char *extname)
 
 #ifdef MITSHM
 static int
-print_mitshm_info(Display *dpy, char *extname)
+print_mitshm_info(Display *dpy, const char *extname)
 {
     int majorrev, minorrev;
     Bool sharedPixmaps;
@@ -970,7 +936,7 @@ print_mitshm_info(Display *dpy, char *extname)
 
 #ifdef XKB
 static int
-print_xkb_info(Display *dpy, char *extname)
+print_xkb_info(Display *dpy, const char *extname)
 {
     int opcode, eventbase, errorbase, majorrev, minorrev;
 
@@ -992,7 +958,7 @@ print_xkb_info(Display *dpy, char *extname)
 #endif
 
 static int
-print_dbe_info(Display *dpy, char *extname)
+print_dbe_info(Display *dpy, const char *extname)
 {
     int majorrev, minorrev;
     XdbeScreenVisualInfo *svi;
@@ -1020,7 +986,7 @@ print_dbe_info(Display *dpy, char *extname)
 }
 
 static int
-print_record_info(Display *dpy, char *extname)
+print_record_info(Display *dpy, const char *extname)
 {
     int majorrev, minorrev;
 
@@ -1032,7 +998,7 @@ print_record_info(Display *dpy, char *extname)
 
 #ifdef XINPUT
 static int
-print_xinput_info(Display *dpy, char *extname)
+print_xinput_info(Display *dpy, const char *extname)
 {
   int           loop, num_extensions, num_devices;
   char          **extensions;
@@ -1040,7 +1006,7 @@ print_xinput_info(Display *dpy, char *extname)
   XExtensionVersion *ext;
 
   ext = XGetExtensionVersion(dpy, extname);
-  
+
   if (!ext || (ext == (XExtensionVersion*) NoSuchExtension))
       return 0;
 
@@ -1092,7 +1058,7 @@ print_xinput_info(Display *dpy, char *extname)
 
 #ifdef XRENDER
 static int
-print_xrender_info(Display *dpy, char *extname)
+print_xrender_info(Display *dpy, const char *extname)
 {
   int		    loop, num_extensions;
   char		    **extensions;
@@ -1111,7 +1077,7 @@ print_xrender_info(Display *dpy, char *extname)
 
   if (!XRenderQueryVersion (dpy, &major, &minor))
     return 0;
-  
+
   print_standard_extension_info(dpy, extname, major, minor);
 
   extensions = XListExtensions(dpy, &num_extensions);
@@ -1202,7 +1168,7 @@ print_xrender_info(Display *dpy, char *extname)
 
 #ifdef COMPOSITE
 static int
-print_composite_info(Display *dpy, char *extname)
+print_composite_info(Display *dpy, const char *extname)
 {
     int majorrev, minorrev, foo;
 
@@ -1218,30 +1184,30 @@ print_composite_info(Display *dpy, char *extname)
 #ifdef PANORAMIX
 
 static int
-print_xinerama_info(Display *dpy, char *extname)
+print_xinerama_info(Display *dpy, const char *extname)
 {
   int              majorrev, minorrev;
 
   if (!XineramaQueryVersion (dpy, &majorrev, &minorrev))
     return 0;
-  
+
   print_standard_extension_info(dpy, extname, majorrev, minorrev);
 
   if (!XineramaIsActive(dpy)) {
     printf("  Xinerama is inactive.\n");
   } else {
-    int i, count = 0; 
+    int i, count = 0;
     XineramaScreenInfo *xineramaScreens = XineramaQueryScreens(dpy, &count);
-    
+
     for (i = 0; i < count; i++) {
       XineramaScreenInfo *xs = &xineramaScreens[i];
-      printf("  head #%d: %dx%d @ %d,%d\n", xs->screen_number, 
+      printf("  head #%d: %dx%d @ %d,%d\n", xs->screen_number,
              xs->width, xs->height, xs->x_org, xs->y_org);
     }
-    
+
     XFree(xineramaScreens);
   }
-  
+
   return 1;
 }
 
@@ -1255,7 +1221,7 @@ static const char *core(DMXInputAttributes *iinfo)
     else                       return "extension";
 }
 
-static int print_dmx_info(Display *dpy, char *extname)
+static int print_dmx_info(Display *dpy, const char *extname)
 {
     int                  event_base, error_base;
     int                  major_version, minor_version, patch_version;
@@ -1311,16 +1277,16 @@ static int print_dmx_info(Display *dpy, char *extname)
                             = XGetExtensionVersion(backend, INAME);
                         if (ext
                             && ext != (XExtensionVersion *)NoSuchExtension) {
-                            
-                            int         count, i;
+
+                            int         dcount, d;
                             XDeviceInfo *devInfo = XListInputDevices(backend,
-                                                                     &count);
+                                                                     &dcount);
                             if (devInfo) {
-                                for (i = 0; i < count; i++) {
+                                for (d = 0; d < dcount; d++) {
                                     if ((unsigned)iinfo.physicalId
-                                        == devInfo[i].id
-                                        && devInfo[i].name) {
-                                        backendname = strdup(devInfo[i].name);
+                                        == devInfo[d].id
+                                        && devInfo[d].name) {
+                                        backendname = strdup(devInfo[d].name);
                                         break;
                                     }
                                 }
@@ -1350,112 +1316,15 @@ static int print_dmx_info(Display *dpy, char *extname)
 
 #endif /* DMX */
 
-#ifdef INCLUDE_XPRINT_SUPPORT
-static
-void print_xprint_attrpool(const char *name, const char *attrpool)
-{
-  int         c;
-  const char *s = attrpool;
-  
-  printf("    %s:\n      ", name);
-  
-  while( (c = *s++) != '\0' )
-  {
-    if (c == '\n') {
-      printf("\n      ");
-    }
-    else
-    {
-      fputc(c, stdout);
-    }
-  }
-  fputc('\n', stdout);
-}
-
-static int
-print_xprint_info(Display *dpy, char *extname)
-{  
-
-  short           majorrev,
-                  minorrev;
-  int             xp_event_base,
-                  xp_error_base;
-  XPPrinterList   printerlist;
-  Screen        **pscreens;
-  int             plcount,
-                  pscrcount,
-                  i;
-
-  if (XpQueryVersion(dpy, &majorrev, &minorrev) == False) {
-    return 0;
-  }
-  
-  print_standard_extension_info(dpy, extname, majorrev, minorrev);
-
-  if (XpQueryExtension(dpy, &xp_event_base, &xp_error_base) == False) {
-    printf("  XpQueryExtension() failed.\n");
-    return 0;
-  }
-  
-  /* Print event info */
-  printf("  xp_event_base=%d, xp_error_base=%d\n", xp_event_base, xp_error_base);
-  
-  /* Print info which screens support the Xprint extension */
-  printf("  Print screens = {");
-  pscreens = XpQueryScreens(dpy, &pscrcount);
-  for( i = 0 ; i < pscrcount ; i++ ) {
-    printf("%s%d", ((i > 0)?(", "):("")), (int)XScreenNumberOfScreen(pscreens[i]));
-  }
-  XFree(pscreens);
-  printf("}\n");
-
-  /* Enumerate the list of printers */
-  printerlist = XpGetPrinterList(dpy, NULL, &plcount);
-  /* Print number of printers, then each printer name and description */
-  printf("  Found %d printers on this server.\n", plcount);
-  for( i = 0 ; i < plcount ; i++) {
-    printf("  printer %d: name='%s', descr='%s'\n",
-           i, NULLSTR(printerlist[i].name), NULLSTR(printerlist[i].desc));
-  }
-  
-  /* Enumerate the list of printers with details */
-  for( i = 0 ; i < plcount ; i++) {
-    char       *printername = printerlist[i].name;
-    XPContext   pcontext;
-    char       *s;
-    
-    printf("  Attributes of printer '%s':\n", NULLSTR(printername));
-
-    pcontext = XpCreateContext(dpy, printername);
-    if (pcontext == None) {
-      printf("    Error: Could not open printer.\n");
-      continue;
-    }
-
-    s=XpGetAttributes(dpy, pcontext, XPJobAttr);     print_xprint_attrpool("XPJobAttr",     s);  XFree(s);
-    s=XpGetAttributes(dpy, pcontext, XPDocAttr);     print_xprint_attrpool("XPDocAttr",     s);  XFree(s);
-    s=XpGetAttributes(dpy, pcontext, XPPageAttr);    print_xprint_attrpool("XPPageAttr",    s);  XFree(s);
-    s=XpGetAttributes(dpy, pcontext, XPPrinterAttr); print_xprint_attrpool("XPPrinterAttr", s);  XFree(s);
-    s=XpGetAttributes(dpy, pcontext, XPServerAttr);  print_xprint_attrpool("XPServerAttr",  s);  XFree(s);
-
-    XpDestroyContext(dpy, pcontext);
-  }
-  
-  XpFreePrinterList(printerlist);
-
-  return 1;
-}
-#endif /* INCLUDE_XPRINT_SUPPORT */
-
 /* utilities to manage the list of recognized extensions */
 
 
 typedef int (*ExtensionPrintFunc)(
-    Display *, char *
+    Display *, const char *
 );
 
 typedef struct {
-    char *extname;
+    const char *extname;
     ExtensionPrintFunc printfunc;
     Bool printit;
 } ExtensionPrintInfo;
@@ -1500,13 +1369,10 @@ static ExtensionPrintInfo known_extensions[] =
 #ifdef DMX
     {"DMX", print_dmx_info, False},
 #endif
-#ifdef INCLUDE_XPRINT_SUPPORT
-    {XP_PRINTNAME, print_xprint_info, False},
-#endif /* INCLUDE_XPRINT_SUPPORT */
     /* add new extensions here */
 };
 
-static int num_known_extensions = sizeof known_extensions / sizeof known_extensions[0];
+static const int num_known_extensions = sizeof known_extensions / sizeof known_extensions[0];
 
 static void
 print_known_extensions(FILE *f)
@@ -1515,7 +1381,7 @@ print_known_extensions(FILE *f)
     for (i = 0, col = 6; i < num_known_extensions; i++)
     {
 	int extlen = strlen(known_extensions[i].extname) + 1;
-	
+
 	if ((col + extlen) > 79)
 	{
 		col = 6;
@@ -1527,7 +1393,7 @@ print_known_extensions(FILE *f)
 }
 
 static void
-mark_extension_for_printing(char *extname)
+mark_extension_for_printing(const char *extname)
 {
     int i;
 
@@ -1569,20 +1435,21 @@ print_marked_extensions(Display *dpy)
     }
 }
 
-static void 
+static void _X_NORETURN
 usage(void)
 {
-    fprintf (stderr, "usage:  %s [options]\n", ProgramName);
-    fprintf (stderr, "-display displayname\tserver to query\n");
-    fprintf (stderr, "-queryExtensions\tprint info returned by XQueryExtension\n");
-    fprintf (stderr, "-ext all\t\tprint detailed info for all supported extensions\n");
-    fprintf (stderr, "-ext extension-name\tprint detailed info for extension-name if one of:\n     ");
+    fprintf (stderr, "usage:  %s [options]\n%s", ProgramName,
+             "-display displayname\tserver to query\n"
+             "-version\t\tprint program version and exit\n"
+             "-queryExtensions\tprint info returned by XQueryExtension\n"
+             "-ext all\t\tprint detailed info for all supported extensions\n"
+             "-ext extension-name\tprint detailed info for extension-name if one of:\n     ");
     print_known_extensions(stderr);
     fprintf (stderr, "\n");
     exit (1);
 }
 
-int 
+int
 main(int argc, char *argv[])
 {
     Display *dpy;			/* X connection */
@@ -1593,18 +1460,32 @@ main(int argc, char *argv[])
 
     for (i = 1; i < argc; i++) {
 	char *arg = argv[i];
-	int len = strlen(arg);
-	
+	size_t len = strlen(arg);
+
 	if (!strncmp("-display", arg, len)) {
-	    if (++i >= argc) usage ();
+	    if (++i >= argc) {
+		fprintf (stderr, "%s: -display requires an argument\n",
+			 ProgramName);
+		usage ();
+	    }
 	    displayname = argv[i];
 	} else if (!strncmp("-queryExtensions", arg, len)) {
 	    queryExtensions = True;
 	} else if (!strncmp("-ext", arg, len)) {
-	    if (++i >= argc) usage ();
+	    if (++i >= argc) {
+		fprintf (stderr, "%s: -ext requires an argument\n",
+			 ProgramName);
+		usage ();
+	    }
 	    mark_extension_for_printing(argv[i]);
-	} else
+        } else if (!strncmp("-version", arg, len)) {
+            printf("%s\n", PACKAGE_STRING);
+            exit (0);
+	} else {
+	    fprintf (stderr, "%s: unrecognized argument '%s'\n",
+		     ProgramName, arg);
 	    usage ();
+	}
     }
 
     dpy = XOpenDisplay (displayname);
